package be.iminds.ilabt.jfed.rspec.model;

import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;

import javax.annotation.Nonnull;
import java.util.*;

/**
* SliverType
*/
public class SliverType {
    private final String name;
    private final List<DiskImage> diskImages;
    private final Integer xenCores;
    private final Integer xenRam;
    private final Integer xenDisk;
    private final List<ExtraXml> other;
    private final List<String> diskImageListRefs;
    private final List<String> flavors;
    private final List<String> flavorListRefs;

//    SliverType(String name) {
//        this.name = name;
//        this.diskImages = new ArrayList<DiskImage>();
//        this.other = new ArrayList<ExtraXml>();
//        this.xenCores = null;
//        this.xenRam = null;
//        this.xenDisk = null;
//    }
//
//    SliverType(String name, Collection<ExtraXml> other) {
//        this.name = name;
//        this.diskImages = new ArrayList<DiskImage>();
//        this.other = other == null ? new ArrayList<ExtraXml>() : new ArrayList<ExtraXml>(other);
//        this.xenCores = null;
//        this.xenRam = null;
//        this.xenDisk = null;
//    }
//
//    SliverType(String name, Collection<DiskImage> diskImages, Collection<ExtraXml> other) {
//        this.name = name;
//        this.diskImages = diskImages == null ? new ArrayList<DiskImage>() : new ArrayList<DiskImage>(diskImages);
//        this.other = other == null ? new ArrayList<ExtraXml>() : new ArrayList<ExtraXml>(other);
//        this.xenCores = null;
//        this.xenRam = null;
//        this.xenDisk = null;
//    }

    //package local: use SliverTypeBuilder
    protected SliverType(String name, Collection<DiskImage> diskImages,
                         Integer xenCores, Integer xenRam, Integer xenDisk,
                         List<String> diskImageListRefs, List<String> flavorListRefs, List<String> flavors,
                         Collection<ExtraXml> other) {
        this.name = name;
        this.diskImages = diskImages == null ? new ArrayList<>() : new ArrayList<>(diskImages);
        this.diskImageListRefs = diskImageListRefs == null ? new ArrayList<>() : new ArrayList<>(diskImageListRefs);
        this.flavorListRefs = flavorListRefs == null ? new ArrayList<>() : new ArrayList<>(flavorListRefs);
        this.flavors = flavors == null ? new ArrayList<>() : new ArrayList<>(flavors);
        this.other = other == null ? new ArrayList<>() : new ArrayList<>(other);
        this.xenCores = xenCores;
        this.xenRam = xenRam;
        this.xenDisk = xenDisk;
    }

    public String getName() {
        return name;
    }

    public List<ExtraXml> getExtraXml() {
        return Collections.unmodifiableList(other);
    }
    public List<DiskImage> getDiskImages() {
        return Collections.unmodifiableList(diskImages);
    }

    public Integer getXenCores() {
        return xenCores;
    }

    public Integer getXenRam() {
        return xenRam;
    }

    public Integer getXenDisk() {
        return xenDisk;
    }

    @Nonnull
    public List<String> getFlavorListRefs() {
        return Collections.unmodifiableList(flavorListRefs);
    }

    @Nonnull
    public List<String> getFlavors() {
        return Collections.unmodifiableList(flavors);
    }

    @Nonnull
    public List<String> getDiskImageListRefs() {
        return Collections.unmodifiableList(diskImageListRefs);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        SliverType that = (SliverType) o;
        return Objects.equals(name, that.name) &&
                Objects.equals(diskImages, that.diskImages) &&
                Objects.equals(xenCores, that.xenCores) &&
                Objects.equals(xenRam, that.xenRam) &&
                Objects.equals(xenDisk, that.xenDisk) &&
                Objects.equals(other, that.other) &&
                Objects.equals(diskImageListRefs, that.diskImageListRefs) &&
                Objects.equals(flavorListRefs, that.flavorListRefs) &&
                Objects.equals(flavors, that.flavors);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, diskImages, xenCores, xenRam, xenDisk, other, diskImageListRefs, flavorListRefs, flavors);
    }

    @Override
    public String toString() {
        return "SliverType{" +
                "name='" + name + '\'' +
                ", diskImages=" + diskImages +
                ", xenCores=" + xenCores +
                ", xenRam=" + xenRam +
                ", xenDisk=" + xenDisk +
                ", other=" + other +
                ", diskImageListRefs=" + diskImageListRefs +
                ", flavorListRefs=" + flavorListRefs +
                ", flavors=" + flavors +
                '}';
    }

    public boolean hasXen() {
        return xenCores != null || xenDisk != null || xenRam != null;
    }
}
